/***************************************************************************
 *   Copyright (C) 2004 by Hans Oischinger                                 *
 *   hans.oischinger@kdemail.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/
#include "edeletecommand.h"

// Core
#include "core/direktorinstance.h"
#include "core/elementdatabase.h"

// Elements
#include "elements/elementnode.h"
#include "elements/elementselection.h"

// Qt
#include <qptrlist.h>

// KDE
#include <klocale.h>

EDeleteCommand::EDeleteCommand()
    : ECommand( i18n( "Delete Elements" ) ),
    m_selection(0)
{
  m_selection = Kast::self()->elementDB().selection().clone();
}

EDeleteCommand::EDeleteCommand( ElementNode * element )
    : ECommand( i18n( "Delete Elements" ) ),
    m_selection(0)
{
  m_selection = new ElementSelection();
  m_selection->append(*element);
}

EDeleteCommand::~EDeleteCommand()
{
  // Now actually delete all elements in m_selection and their children
  QPtrListIterator<ElementNode> itr( m_selection->elements() );
  for( ; itr.current(); ++itr )
  {
    Kast::self()->elementDB().deleteEntry( itr.current() );
  }

  delete m_selection;
}

void EDeleteCommand::execute( )
{
  // remove the Elements from the ObjectDB
  QPtrListIterator<ElementNode> itr( m_selection->elements() );
  for( ; itr.current(); ++itr )
  {
    Kast::self()->elementDB().removeEntry( itr.current() );
  }
}

void EDeleteCommand::unexecute( )
{
  // set back old selection
  Kast::self()->elementDB().selection() = *m_selection;
  // revive the Elements through the ObjectDB
  QPtrListIterator<ElementNode> itr( m_selection->elements() );
  for( ; itr.current(); ++itr )
  {
    Kast::self()->elementDB().reviveEntry( itr.current() );
  }
}



