/***************************************************************************
 *   Copyright (C) 2004 by Hans Oischinger                                 *
 *   hans.oischinger@kdemail.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/
#ifndef GRADIENT_H
#define GRADIENT_H

#include "paintserver.h"

class AnimatedTransformList;
class AnimatedEnum;
class URIReference;
class KRenderingPaintServer;
/**
Abstract base class for all gradients (LinearGradient, RadialGradient)
 
@author Hans Oischinger
*/
class Gradient : public PaintServer
{
  Q_OBJECT
public:

  enum GradientUnits {
    GRADUNITS_OBJECTBOUNDINGBOX,  // default
    GRADUNITS_USERSPACEONUSE,
  };

  enum SpreadMethod {
    SPREADMETHOD_UNKNOWN = 0,
    SPREADMETHOD_PAD     = 1,
    SPREADMETHOD_REFLECT = 2,
    SPREADMETHOD_REPEAT  = 3
  };

  Gradient();
  Gradient( const Gradient& );
  ~Gradient();

  Gradient &operator=(const Gradient&);
  Gradient &operator|=(const Gradient&);

  virtual Attribute* getAttrByName( const QString& name );
  virtual Element::Hit hits( const QPoint & );
  virtual QRect bbox( KCBBoxFlags flags = BBOX_DEFAULT ) const;

  URIReference& ref() const { return *m_ref; }

  AnimatedTransformList& gradientTransformList( bool inherited=false ) const;
  AnimatedEnum& gradientUnits( bool inherited=false ) const;
  AnimatedEnum& spreadMethod( bool inherited=false ) const;

  KRenderingPaintServer* context() const { return m_context; }
  void setContext( KRenderingPaintServer* context );

protected:
  URIReference *m_ref;
  AnimatedTransformList* m_animTransList;
  AnimatedEnum* m_gradientUnits;
  AnimatedEnum* m_spreadMethod;

  KRenderingPaintServer* m_context;
};

#endif
