/*  This file is part of the KDE project
    Copyright (C) 2007 Matthias Kretz <kretz@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#ifndef PHONON_MIXERELEMENT_H
#define PHONON_MIXERELEMENT_H

#include "phonon_mixer_export.h"

template<typename T> class QList;
class QObject;
class QIcon;
class QStringList;

namespace Phonon
{
namespace Mixer
{

class MixerElementPrivate;
class MixerEnumControl;
class MixerIntegerControl;
class MixerToggleControl;
class MixerControl;
class PHONON_MIXER_EXPORT MixerElement
{
    Q_DECLARE_PRIVATE(MixerElement)
    public:
        const QObject *notifier() const;
        operator const QObject *() const { return notifier(); }

        MixerElement(const MixerElement &rhs);
        MixerElement &operator=(const MixerElement &rhs);
        bool operator==(const MixerElement &rhs) const;
        bool operator!=(const MixerElement &rhs) const { return !operator==(rhs); }
        bool operator< (const MixerElement &rhs) const;
        bool operator> (const MixerElement &rhs) const { return !operator<(rhs) && !operator==(rhs); }
        bool operator<=(const MixerElement &rhs) const { return operator<(rhs) || operator==(rhs); }
        bool operator>=(const MixerElement &rhs) const { return !operator<(rhs); }

        bool isValid() const;

        QString name() const;
        QIcon icon() const;
        QList<MixerControl> allControls() const;
        QList<MixerToggleControl> toggleControls() const;
        QList<MixerIntegerControl> integerControls() const;
        QList<MixerEnumControl> enumControls() const;
        MixerToggleControl addToggleControl();
        MixerIntegerControl addIntegerControl(int minimumValue, int maximumValue, int step = 1);
        MixerEnumControl addEnumControl(const QStringList &options);
        bool removeControl(const MixerControl &);

    protected:
        MixerElementPrivate *const d_ptr;
};
} // namespace Mixer
} // namespace Phonon

#endif // PHONON_MIXERELEMENT_H
